from xml.dom import minidom
from os import path
from os import getenv
from sys import stdout

class XmlConfig:
    __configPath = None
    __rootElement = None
    __options = None

    def __init__(self):
        if not self.__rootElement:
            xmlFile = path.join(self.getConfigPath(), "fgen.xml")
            doc = minidom.parse(xmlFile)
            self.__rootElement = doc.documentElement

    def getConfigPath(self):
        if not self.__configPath:
            home = getenv("HOME")
            if home == None:
                exit("Error: you need to set HOME environment variable")
            self.__configPath = path.join(home, ".fgen")
        return self.__configPath

    def getAuthor(self):
        return self.__getSingleValue(self.__rootElement, "author")

    def getWrapperPrefix(self):
        return self.__getSingleValue(self.__rootElement, "wrapper_prefix")

    def getCppSuffix(self):
        suffix = self.__getSingleValue(self.__rootElement, "default_cpp_suffix")
        return ".cc" if suffix == None or suffix == "" else suffix

    def getUnitTestDir(self):
        return self.__getSingleValue(self.__rootElement, "unit_test_dir")
    
    def get(self, key):
        return self.__getSingleValue(self.__rootElement, key)

    def getOptions(self):
        if not self.__options:
            self.__options = []
            nodes = self.__rootElement.getElementsByTagName("options")        
            if len(nodes) != 1:
                raise ValueError("Error: there are less or more than 1 node for options")
            
            nodes = nodes[0].getElementsByTagName("option")
            for node in nodes:
                shortOpt = self.__getSingleValue(node, "short_option")
                longOpt = self.__getSingleValue(node, "long_option")
                hasArg = self.__getSingleValue(node, "has_argument")
                package = self.__getSingleValue(node, "package")
                slotClass = self.__getSingleValue(node, "slot_class")
                parseOrder = self.__getSingleValue(node, "parse_order")
                comment = self.__getSingleValue(node, "comment")
                self.__options.append((shortOpt, longOpt, hasArg, package, slotClass, parseOrder, comment))
            self.__options.sort(self.__optionCmp)
        return self.__options
    
    def getShortOptions(self):
        shortOptions = ""
        options = self.getOptions()
        for option in options:
            shortOpt = option[0]
            hasArg = option[2]
            shortOptions = shortOptions + shortOpt
            if hasArg.lower() == "true":
                shortOptions = shortOptions + ':'
        return shortOptions
    
    def getLongOptions(self):
        longOptions = []
        options = self.getOptions()
        for option in options:
            longOpt = option[1]
            hasArg = option[2]
            if hasArg.lower() == "true":
                longOpt = longOpt.strip() + '='
            longOptions.append(longOpt)
        return longOptions
    
    def printOptionUsage(self, output = stdout):
        print>>output, "Options:"
        for option in self.getOptions():
            shortOpt = option[0]
            longOpt = option[1]
            hasArg = option[2]
            comment = option[6]
            if hasArg.lower() == "true":
                print>>output, "  -%s %s, --%s=%s" % (shortOpt, longOpt.upper(), longOpt, longOpt.upper())
            else:
                print>>output, "  -%s, --%s" % (shortOpt, longOpt)
            print>>output, "      %s" % comment

    def __getValueList(self, element, name):
        values = []
        nodes = element.getElementsByTagName(name)
        for node in nodes:
            if node.nodeType == node.ELEMENT_NODE:
                for nd in node.childNodes:
                    if nd.nodeType == node.TEXT_NODE or nd.nodeType == node.CDATA_SECTION_NODE:
                        values.append(nd.data)
        return values

    def __getSingleValue(self, element, name):
        nodes = element.getElementsByTagName(name)
        if len(nodes) != 1:
            raise ValueError("Error: there are less or more than 1 node for %" % name)

        if nodes[0].nodeType == nodes[0].ELEMENT_NODE:
            for node in nodes[0].childNodes:
                if node.nodeType == node.TEXT_NODE or node.nodeType == node.CDATA_SECTION_NODE:
                    return node.data
        return ""
    
    def __optionCmp(self, x, y):
        po1 = eval(x[5])
        po2 = eval(y[5])
        return 1 if po1 > po2 else 0 if po1 == po2 else -1
